#!/usr/bin/env python3

# from {proj}.version import __version__
from fire import Fire
import threading
import time
from console import fg,bg,fx
import re
from cronvice import mmapwr
#import readchar
from sshkeyboard import listen_keyboard, stop_listening

global_key = ""

def get_global_key():
    global global_key
    a = global_key
    #global_key = ""
    return a

# https://sshkeyboard.readthedocs.io/en/latest/reference.html#sshkeyboard.listen_keyboard_manual
# https://github.com/ollipal/sshkeyboard
#
# MMAP VARIANT =============================================
#
class MmapSimulatedKeyboard(threading.Thread):
    def __init__(self,  name='mmap-input-thread', ending=None):
        '''
        Restart itself as thread
        '''
        self.block = False
        self.ending = ending
        self.cursor = 0


        self.history = {} # dict with past commands
        self.history_pointer = 0
        super(MmapSimulatedKeyboard, self).__init__(name=name)
        self.start()

    def get_global_key(self):
        '''
        use to read the situation and reset on enter from the main program, True when ENTER
        '''
        global global_key
        if global_key.find("\n")>=0:
            a = global_key
            global_key = ""
            return a, True, (a,"")
        return global_key, False, (global_key,"")


    def run(self):
        '''
        this is a mandatory for Thread child
        '''
        global global_key

        self.t = threading.current_thread()

        while True:
            #print("!...  ........................keyboard listen START")
            time.sleep(.5)
            res = mmapwr.mmread_n_clear()
            #
            # works here
            #mmapwr.mmwrite(res, mmapwr.MMAPRESP)
            #
            # earlier, tuple was sent.....
            #print("###", res, type(res) )
            if type(res)==tuple:
                global_key=res[0]
            else:
                global_key = res # maybe some operation is needed?
            #print("!...  ........................keyboard listen STOP")
            #print("D...",f"/{global_key}/")
            if global_key.strip() == self.ending:
                #print("!...  ........................keyboard listen BREAK")
                break
        #print("!...  ........................keyboard THREAD  ENDED")


#
#
#   ======================= SSH KEYBOARD VARIANT ===============
#
class KeyboardThreadSsh(threading.Thread):
    def __init__(self,  name='keyboard-input-thread', ending=None):
        '''
        Restart itself as thread
        '''
        self.block = False
        self.ending = ending
        self.cursor = 0


        self.history = {} # dict with past commands
        self.history_pointer = 0
        super(KeyboardThreadSsh, self).__init__(name=name)
        self.start()


    def text_split(self):
        '''
        operates on global_key and cursor, returns 3 parts
        0 is before 1.st letter
        '''
        global global_key
        a,b,c="","",""
        lel = len(global_key)

        cursor = self.cursor

        if  cursor>lel:
            cursor=lel
        if cursor<0:
            cursor=0

        if lel==0:
            return a,b

        #  a b c
        # 0 1 2 3
        if cursor==lel:
            a=global_key
            return a,b

        if cursor == 0:
            b = global_key
            return a,b

        if cursor == 1:
            a = global_key[0]
            b = global_key[1:]
            return a,b
        else:
            a=global_key[:cursor]
            b=global_key[cursor:]
        return a,b


    def press(self,key):
        '''
        the callback - uses the global_key to remember
        '''

        global global_key

        #key2 = key.encode("utf8")
        ansi_esc = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
        key2 = ansi_esc.sub('', key)

        #print("D...", f"'{key2}' pressed", flush=True)
        #print("D...", f"/{global_key}/ , cu=={self.cursor} :",  flush=True ) #self.text_split(),
        # if global_key.find(";")>0 and key2=="t":
        #     #pass
        #     print(">>>>>>>>>>>>killhere")
        #     global_key = ""
        #     #return

        nx=0
        #print("XXX",nx,flush=True)
        nx+=1
        # the tricks: space up down
        if key2=="space":
            key2=" "
            #self.cursor+=1 # not here ...
        elif key2=="delete":
            #print("XXX",nx,flush=True)
            nx+=1
            if self.cursor<=len(global_key):
                global_key =  global_key[:self.cursor]+global_key[self.cursor+1:]
            key2=""
        elif key2=="backspace":
            #print("XXX",nx,flush=True)
            nx+=1
            global_key =  global_key[:self.cursor-1]+global_key[self.cursor:]
            self.cursor-=1
            key2=""
        elif key2=="end":
            self.cursor = len(global_key)
            key2=""
        elif key2=="home":
            self.cursor = 0
            key2=""
        elif key2=="left":
            self.cursor-=1
            key2=""
        elif key2=="right":
            self.cursor+=1
            key2=""
        elif key2=="enter":
            key2="\n"
            self.cursor=len(global_key) # there is a reason
            n = len(self.history)
            self.history[n] = global_key
            self.history_pointer = n+1
            # DEBUG - track history
            # print(self.history, self.history_pointer)
        elif key2=="up":
            key2=""
            if self.history_pointer>0:
                self.history_pointer-=1
                global_key = self.history[self.history_pointer]
                self.cursor=len(global_key)
        elif key2=="down":
            key2=""
            if self.history_pointer<len(self.history)-1:
                self.history_pointer+=1
                global_key = self.history[self.history_pointer]
                self.cursor=len(global_key)
            elif self.history_pointer==len(self.history)-1:
                #print("D... EDGE")
                self.history_pointer+=1 # empty
                global_key = ""
                self.cursor=0
        elif key2=="tab":
            key2=""
        #print("XXY",nx,flush=True)
        nx+=1
        #else:
        #    self.cursor+=1

        # print("D...  ..........",self.cursor, "after press /    limit",len(global_key)+1)

        #### global_key = f"{global_key[:self.cursor+1]}{key}{global_key[self.cursor+1:]}" # KEEP IT GLOBAL !!!!!
        a,b = self.text_split()
        #print("XXY",nx,flush=True)
        nx+=1
        # print(f"i... joining {a} {key} {b}")
        global_key = f"{a}{key2}{b}" # KEEP IT GLOBAL !!!!!
        #print("XXz5",nx,flush=True)
        nx+=1
        if len(key2)>0:
            self.cursor+=1

        #print("XXz4",nx)
        nx+=1
        # correct cursor if  it went too far
        if self.cursor>len(global_key)+1:self.cursor=len(global_key)+1
        #print("XXz3",nx)
        nx+=1
        if self.cursor<0:self.cursor=0
        #print("XXz2",nx)
        nx+=1

        #print("D...",f"RESULT: '{global_key}' , cu=={self.cursor} :", self.text_split() )
        #self.cursor+=1

        if key2=="\n":
            stop_listening()
        #print("XXzzz",nx,flush=True)
        nx+=1



    def get_global_key(self):
        '''
        use to read the situation and reset on enter from the main program, True when ENTER
        '''
        global global_key
        if global_key.find("\n")>=0:
            a = global_key
            global_key = ""
            return a, True, self.text_split()
        return global_key, False, self.text_split()


    def run(self):
        '''
        this is a mandatory for Thread child
        '''
        global global_key

        self.t = threading.current_thread()

        while True:
            #print("!...  ........................keyboard listen START")
            listen_keyboard(on_press=self.press,
                            debug = False,
                            delay_second_char=0.1,
                            delay_other_chars=0.1,
                            lower = False, # IT WAS DEFAULT
                            sequential=True) # syncio
            #print("!...  ........................keyboard listen STOP")
            #print("D...",f"/{global_key}/")
            if global_key.strip() == self.ending:
                print("D...  ........................keyboard listen got BREAK...")
                time.sleep(0.7) # give time to main to exit
                break
        #print("!...  ........................keyboard THREAD  ENDED")




#--------https://stackoverflow.com/questions/2408560/python-nonblocking-console-input
# ON ENTER
# CALL WITH CALLBACK FUNCTION AS A PARAMETER
#class KeyboardThread(threading.Thread):




def main():
    print()

if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3
"""
 Purpose is to have defined objects - LOCAL commands would work on these objects.
https://realpython.com/python-classes/#special-methods-and-protocols
#
https://stackoverflow.com/questions/12101958/how-to-keep-track-of-class-instances
#
each class keeps its own instances
"""
from fire import Fire
from weakref import WeakSet  # --------- weakset doeasnt survive  creation in other MOD and forgetting!!!!
from console import fg, bg
import itertools
import os
import pandas as pd
import datetime as dt

class O_object(object):
    #instances = None
    #name = None
    #typ = None
    #@classmethod
    #def add_to_instance(cls, name, obj):
    #    O_object.instances[name] = obj

    def __new__(cls,name):
        """
        Keep list of instances (breaks with O_dataframes)
        Do not create existing name - return None
        """
        #print("NEW HERE")
        instance = object.__new__(cls)
        if "instances" not in cls.__dict__:
            #cls.instances = WeakSet()  # LIST SURVIVES MORE across  MODULES
            cls.instances = [] # WeakSet()
        names = [obj.get_name() for obj in cls.instances]
        if name in names:
            print(f"X... {fg.red}object /{name}/ already exists ... NOT CREATED{fg.default}")
            return None
        #cls.instances.add(instance)  # LIST SURVIVES MORE
        cls.instances.append(instance)
        return instance

    @classmethod
    def get_instances(cls):
        """
        works same as  .instances
        """
        return list(cls.instances) #Returns list of all current instances


    def __init__(self, name):
        #print("INIT_HERE", self.instances)
        self.name = name
        self.typ = "o" # useful when reading files o_  h_
        self.df_data = None
        self.src_filename = None # name of the source file
        self.src_basename = None # name of the source file
        self.src_path = None # path of the source_file

        self.started = None
        self.elements = None
        self.comment = ""

    def __str__(self): #dunder methods
        ln = 16-len(self.name)
        ss = " "*ln
        col = fg.white
        if self.typ == "d": col = fg.red
        if self.typ == "h": col = fg.green
        return f"s... Object:  {fg.green}{self.name}{fg.default} {ss} ... type {col}{self.typ} {fg.default}... {hex(id(self))}{fg.default}"

    #--------- I like to see the address.....
#    def __repr__(self): # for debugging.... they say
#        return f"{type(self).__name__}(\"{self.name}\")"

    def get_name(self):
        return self.name

    def get_type(self):
        return self.typ

    def read_from_file(self,filename):
        """
        fill pandas dataframe - for spectrum and for asc
        self.pd_data    defined
        """
        base = os.path.splitext(filename)[0]
        potential_name = f"{self.typ}_{base}"
        if object_exists(potential_name):
            print(f"X... {fg.red}{potential_name} REPLACING object {fg.default}")
            o = get_object(potential_name)
            self.instances.remove(o)
            #del o
            #return False
        self.name = f"{self.typ}_{base}"
        self.src_filename = filename
        self.src_basename = os.path.basename(filename)
        self.src_path = os.path.dirname(filename)
        print(f"i... opening file /{fg.yellow}{filename}{fg.default}/ and naming /{fg.green}{self.name}{fg.default}/")
        # - - -from ascgeneric,
        wastart = dt.datetime.now()
        self.df_data = pd.read_csv(filename, comment = "#", header=None,  sep = r"\s+" )
        wastop = dt.datetime.now()-wastart
        #
        print(f"i... {bg.white}{fg.green} ... LOADED {len(self.df_data)/1000/1000:.2f} Mrows ... {len(self.df_data)/wastop.total_seconds()/1000/1000:.2f} Mrows/sec ", end="")
        print( " "*40,f"{fg.default}{bg.default}")

        #print(f"i... ... ... ... ... ... ... data loaded. LEN={len(self.df_data)}")
        return True


    @classmethod
    def from_file(cls, filename):
        """
         # create new object from something ...  O_object.create_from()
        see read_from_file
        """
        #names = [obj.get_name() for obj in cls.instances]
        #potential_name = "a"#f"{cls.typ}_{base}"
        print(cls)
        print( "typ" in cls.__dict__ )
        #if object_exists(potential_name):
        #    return None
        a = cls("tmp")
        if a is None:
            return None
        res = a.read_from_file(filename)
        if res: return a
        return None

    @staticmethod
    def abouts():
        print(f"h... About: this is a static method of either O_object or its daugters..." )

    @classmethod
    def about(cls):
        """
        # access to class name
        """
        print(f"h... About: this is a class {cls.__name__}...")

# ================================================ ******************************************************************************
# ================================================ ******************************************************************************
# ================================================ ******************************************************************************


class O_histogram(O_object):
    def __init__(self, name):
        super().__init__(name)
        self.typ = "h"

#    def __repr__(self):
#        return f"O_histogram({self.name})"

# ================================================ ******************************************************************************
# ================================================ ******************************************************************************
# ================================================ ******************************************************************************

class O_dataframe(O_object):
    def __init__(self, name):
        super().__init__(name)
        self.typ = "d"

    def printme(self):
        if self.df_data is not None:
            #print(f"D... {type(self.df_data)} exists...")
            if len(self.df_data)>2:
                print( self.df_data.iloc[[0,1,-2,-1]] )
            else:
                print( self.df_data)
        #print("D... end of printme" )





# ================================================ ******************************************************************************
# ================================================ ******************************************************************************
# ================================================ ******************************************************************************

def list_objects():
    """
    LIST and print
    """
    ok = False
    # I do not use weakset
    for i in get_objects_list( DEBUG=False):# WeakSet(itertools.chain(   O_dataframe.instances,O_histogram.instances   )):
        print(i)
        ok = True
    if not ok: print("-... no objects in memory")

def get_objects_list_names():
    SS = get_objects_list(DEBUG=False)
    names = [obj.get_name() for obj in SS]
    return names

def get_objects_list( DEBUG=True):
    """
    LIST OF histograms and dataframes together
    """
    # check of there are already objects in the CLASSes
    #print("D... instances...")
    Ad = "instances" in O_dataframe.__dict__
    Bh = "instances" in O_histogram.__dict__

    if DEBUG: print(f"D... {fg.dimgray}A x B ... ", Ad,Bh , fg.default)

    if Ad and Bh:
        return list(O_dataframe.instances) + list(O_histogram.instances)
    elif Ad:
        return list(O_dataframe.instances)
    elif Bh:
        return list(O_histogram.instances)
    #else:
    #    yield []
    return []
    #  I do not use weak-set
    # # LISTS --------
    # if A and (type(O_dataframe.instances) != WeakSet) and B:
    #     return O_dataframe.instances + O_histogram.instances
    # if A and (type(O_dataframe.instances) != WeakSet) :
    #     return O_dataframe.instances
    # if B and (type(O_histogram.instances) != WeakSet) :
    #     return O_histogram.instances
    # # --------- WeakSet
    # if A and B :
    #     #LI = list( WeakSet(itertools.chain(   O_dataframe.instances,O_histogram.instances   )) )
    #     LI = list( itertools.chain(   O_dataframe.instances,O_histogram.instances   ))
    #     #print(LI)
    #     return LI
    # if A  :
    #     return list( WeakSet(itertools.chain(   O_dataframe.instances   )) )
    # if B  :
    #     return list( WeakSet(itertools.chain(   O_histogram.instances   )) )
    # return []


def object_exists( name ):
    """
    check LIST OF histograms and dataframes together
    """
    SS = get_objects_list( DEBUG=False)
    #print( type(SS) )
    #print( SS )

    names = [obj.get_name() for obj in SS]
    if name in names: return True
    return False

def get_object( name ):
    """
    get one
    """
    #print("D...  in get_object:", name)
    SS = get_objects_list(DEBUG=False)
    #print(f"D...   {SS}")
    #print(f"D...   {len(SS)}")
    #print("D... FOR     ================")
    for i in list(SS):
        #print( "D... ?", i.get_name()  )
        if name == i.get_name():
            #print("D... OUTSIDE ================")
            return i
    #print("D... OUTSIDE ================")
    return None

# ================================================ ******************************************************************************
# ================================================ ******************************************************************************
# ================================================ ******************************************************************************

def main():
    """
    testing different variants
    """
    print( get_objects_list(DEBUG=True) )
    print()
    o1 = O_object("o1")
    o2 = O_object("o1")

    h1 = O_histogram("h1")
    h2 = O_histogram.from_file("hpge_b0c00.asc")

    d1 = O_dataframe("d1")
    d2 = O_dataframe("d2")

    print("-"*50, 'all defined')

    #print(o1)
    print(h1)
    print(h2)
    print(d1)

    print("repr..h1",repr(h1))
    print("c... call:", h1.get_name() ,"X", h1.get_type() )
    print("c... call:", getattr(h1,"name") ,"X", getattr(h1,"typ") )
    print("c... call:", h1.name ,"X", h1.typ )
    print("c... call:", d1.name ,"X", d1.typ )
    print( "d1repr... ",repr(d1) )

    print("-"*50," print some d1 ")
    h1.about()
    h2.about()
    h2.abouts()
    d1.about()


    print("-"*50," LISTING INSTANCES -  every class has its own list")
    # for i in  O_histogram.instances:
    #     print("#Histo#  printed name: ",i.get_name(), i.get_type() )
    #     i.about()
    # print()

    # for i in  O_object.get_instances():
    #     print("~Object~  printed name: ",i.get_name(), i.get_type() )
    #     i.about()
    # print()

    # for i in O_dataframe.get_instances():
    #     print("`Datafr`  printed name: ",i.get_name(), i.get_type() )
    #     print( repr(i))
    #     i.about()

    # print(  "i...  instances are of type :",  type(O_dataframe.instances)  )
    # # if type(O_dataframe.instances) == "<class '_weakrefset.WeakSet'>":
    # if type(O_dataframe.instances) != WeakSet:
    #     print("-"*50," LISTING SUM OF INSTANCES")
    #     for i in O_dataframe.instances+O_histogram.instances:
    #         print(i)

    list_objects()



    #print(O_histogram.instances) # CRASHES, only classmethod can be called

if __name__ == "__main__":
    Fire(main)



import sys
sys.path.append(".")         # can find version in . by pytest
sys.path.append("./cronvice")  # can find version from Jupyter
#from version import __version__
# try:
#     from cronvice.version import __version__
# except:
#    print("")



#!/usr/bin/env python3
"""
 These modules should contain one function each...


"""

from fire import Fire
from console import fg,bg
from cronvice import config
from cronvice import objects

KNOWN_COMMANDS_LOCAL_TYPE = "OBJ"

def main(*args,**kwargs):
    #print(f"{fg.dimgray}D... main() @fn_lo: args/kwargs.../{args}/{kwargs}/{fg.default}")
    #print(f"D... main() @fn_show: args/kwargs.../{args}/{kwargs}/")
    if len(args)==0:
        print("D...  give me an object: allowed objects:",objects.get_objects_list_names() )
        return
    oname = args[0]
    if objects.object_exists(oname):
        print(f"{fg.green}i... showing {oname}{fg.default}")
        obj = objects.get_object( oname )
        print(f"    filename: {obj.src_filename}")
        print(f"    basename: {obj.src_basename}")
        print(f"    path    : {obj.src_path}")
        print(f"    type    : {obj.typ}")
        #if obj.started is not None:
        print(f"    started : {obj.started}")
        #if obj.elements is not None:
        print(f"    elements: {obj.elements:,d}")
        #if obj.comment is not None:
        print(f"    comment : {obj.comment}")
    else:
        print(f"i... {fg.red} NOT showing {oname}{fg.default}")


if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3
"""
 These modules should contain one function each...

MODIFIY THE original cronvice adding these functions only + editing CONFIG

... they need some common connect point to have common objects:
 1. config ?
  -  yes, but is there possible to do without?
     - ? for the sake of cronvice initial paradigm? or is that ok?

"""

from fire import Fire
from console import fg,bg
from cronvice import config
from cronvice import objects
import os
# I NEED TO UPDATE  -   config.object_list

KNOWN_COMMANDS_LOCAL_TYPE = "FS"

def main(*args,**kwargs):
    #print(f"{fg.dimgray}D... main() @fn_load: args/kwargs.../{args}/{kwargs}/{fg.default}")
    if len(args)==0:
        print(f"X... {fg.red}give me a file as a parameter...{fg.default}")
        return None
    # ===== loading ------*
    fname = args[0]
    print(f"i... {fg.green}loading {fname}{fg.default}")
    if not os.path.exists(fname):
        print("X... {fg.red}doesnt exist...{fg.default}")
        return

    ext = os.path.splitext(fname)[-1]
    #d1 = objects.O_dataframe("dodo")
    #h1 = objects.O_histogram("hoho")
    #print( objects.get_objects_list() )
    # ==============================================
    if ext == ".asc":
        objects.O_dataframe.from_file(fname)
    elif ext == ".txt":
        objects.O_histogram.from_file(fname)
    else:
        print(f"{fg.red}X...  unknown extension /{ext}/ ... not loading {fg.default} ")
    #objects.get_objects_list().append( args[0] )
    objects.list_objects()


if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3

from fire import Fire
from console import fg, bg
import shutil
import os
import datetime as dt
import shlex
import subprocess as sp
#import config
from cronvice import config
import time

#=========================================================================
#
#-------------------------------------------------------------------------
#

def is_session_desktop():
    if os.environ.get('SSH_TTY'):
        return False #"SSH Terminal Session"
    elif os.environ.get('DISPLAY'):
        return True#"Desktop Session"
    else:
        return False#"Unknown Session"

#=========================================================================
#
#-------------------------------------------------------------------------
#

def enter_screen(scrname, term=""):
    """
    it enters when run from here
    """
    while True:
        sessions = list_screen_sessions()
        # ADD .local/bin
        env = os.environ.copy()
        env['PATH'] = env.get('PATH', '') + ':' + os.path.expanduser('~/.local/bin')
        #print(sessions)
        #print(scrname)
        startingwait = dt.datetime.now().strftime("%H:%M:%S")
        if is_in_screen(scrname, sessions):
            CMD = f"screen -x {scrname}"
            if is_session_desktop() and term != "same":
                args = shlex.split(f"xterm -e bash -c '{CMD}'")
                process = sp.Popen(args,  stdout=sp.DEVNULL, stderr=sp.DEVNULL, env=env)
                process.poll()
                break
            else:
                args = shlex.split(f"bash -c '{CMD}'")
                res = sp.call(args, env=env)
                break
        else:
            print(f"i... waiting to enter {scrname} ...  {startingwait} =>  {dt.datetime.now().strftime("%H:%M:%S")}", end="\r")
            time.sleep(2)

#=========================================================================
#
#-------------------------------------------------------------------------
#

def stop_screen(scrname):
    """
    it enters when run from here
    """
    sessions = list_screen_sessions()
    # ADD .local/bin
    env = os.environ.copy()
    env['PATH'] = env.get('PATH', '') + ':' + os.path.expanduser('~/.local/bin')
    #print(sessions)
    #print("-", scrname, "-")
    if is_in_screen(scrname, sessions):
        CMD = f"screen -X -S {scrname} quit"
        if is_session_desktop():
            args = shlex.split(f"xterm -e bash -c '{CMD}'")
            process = sp.Popen(args,  stdout=sp.DEVNULL, stderr=sp.DEVNULL, env=env)
            process.poll()
        else:
            args = shlex.split(f"bash -c '{CMD}'")
            res = sp.call(args, env=env)
    else:
        print("X.. no such screen:", scrname)



#=========================================================================
#
#-------------------------------------------------------------------------
#

def list_screen_sessions():
    """
    return the existing screen sessions; used in enter and stop....
    """
    try:
        result = sp.run(['screen', '-ls'], capture_output=True, text=True, check=True)
        #print(result.stdout)
        res = result.stdout.strip().split('\n')[1:-1]
        #print(res) # many
        return res
    except sp.CalledProcessError as e:
        #print(f"x... screen ls - error occurred: {e}")
        pass
    return None


#=========================================================================
#
#-------------------------------------------------------------------------
#

def tidy_screen_sessions( sessions):
    """
    tidy uo  the list of existing screen sessions; decodes also time
    """
    if sessions is None:return None
    #print('tidy', sessions) #many
    running = []
    for i in sessions:
        #print(i)
        line = i.strip().split("\t")
        xtime = line[1].strip("(").strip(")")
        xtime = dt.datetime.strptime(xtime, "%d/%m/%y %H:%M:%S")
        #tses.append(f"{line[0]}  {xtime}")
        xlong = dt.datetime.now() - xtime
        name_to_app = line[0].split(".")[-1]
        # remove _CX
        if name_to_app.find( config.CONFIG['screentag'] ) > 0:
            print(f"{line[0]:20s} -  {xtime}  -  {xlong}")
            name_to_app = name_to_app.split( config.CONFIG['screentag'] )[0]
            running.append(name_to_app )
        #print('runinng', running)
    return running



#=========================================================================
#
#-------------------------------------------------------------------------
#

def is_in_screen(TAG, sessions):
    """
    if tag in screen list => True
    """
    if sessions is None:return False
    for i in sessions:
        #print(i, TAG, i.find(TAG) )
        if i.find(f"{TAG}{config.CONFIG['screentag']}") > 0:
            return True
    return False

#=========================================================================
#
#-------------------------------------------------------------------------
#


def del_job_anycommand( cron, tag):
    """
    older
    """
    ACT = False
    RMTG = f"screen -dmS {tag} " #SPACE IMPORTANT
    for job in cron:
        if job.command.find(RMTG) > 0:
            print(f"i... removing /{RMTG}/ ")#... {job}")
            cron.remove(job)
            ACT = True
    if ACT:
        cron.write()

if __name__ == "__main__":
    Fire({"e": enter_screen
        }
         )



#!/usr/bin/env python3
"""
 These modules should contain one function each...
ls for the objects...

"""

from fire import Fire
from console import fg,bg
from cronvice import config
from cronvice import objects

KNOWN_COMMANDS_LOCAL_TYPE = "OTHER"

def main(*args,**kwargs):
    #print(f"{fg.dimgray}D... main() @fn_lo: args/kwargs.../{args}/{kwargs}/{fg.default}")
    #print( objects.get_objects_list() )
    objects.list_objects()
    #print("D... lo ends here")

if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3
from fire import Fire
from crontab import CronTab  # install python-crontab
from console import fg, bg
import shutil
import os
import datetime as dt
import shlex
import subprocess as sp
from cronvice import libs_screen
from cronvice import config
from cronvice.config import mycron
import time
import sys



#=========================================================================
#
#-------------------------------------------------------------------------

def reschedule_to_1m(tag, every="1m"):
    """
    """
    print("D... deleting job first", tag)
    del_job(tag) # ment to remove 31.2. job
    print(f"D... adding job second: tag== {tag}  for every /{every}/")
    add_job(tag, every)



#=========================================================================
#
#-------------------------------------------------------------------------

def set_to_never(tag):
    """
    """
    ACT = False
    #file_path = os.path.abspath(__file__)
    file_path = os.path.abspath(sys.modules['__main__'].__file__)

    # RMTG = f"screen -dmS {tag} " #SPACE IMPORTANT
    RMTG = f"{file_path}"
    for job in config.mycron:
        #print(f"/{job.command}/..")
        if job.command.find(RMTG) > 0 and job.command.endswith(f" r {tag}") > 0: # no space
            print(f"i... setting to never /{RMTG}/{tag}/ ")#... {job}")
            #config.mycron.remove(job)
            #config.mycron.remove(job)
            job.day.on(31)
            job.hour.on(23)
            job.minute.on(59)
            job.month.on('FEB')
            ACT = True
    if ACT:
        config.mycron.write()
    else:
        print("X... nothing set")

#=========================================================================
#
#-------------------------------------------------------------------------

def just_list_cron( running=None):
    """
    aas it says. But I like to have clean list....
    """
    env = os.environ.copy()
    CMD = "crontab -l"
    args = shlex.split(CMD)
    # CRASH ON NO CRONTAB DEFINED
    ok = False
    ret = ""
    try:
        ret = sp.check_output(args)#Popen(args, env=env, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
        ok = True
    except:
        pass
    if not ok:
        print("X... no crontab")
        return
    ret = ret.decode("utf8").strip().split("\n")
    ret = [ x.strip() for x in ret if str(x).find("#") != 0 ]
    ret = [ x.strip() for x in ret if len(str(x).strip(" ")) != 0  ]
    #

    #print(running)
    for i in ret:
        #process.poll()
        j = i.split(" ")
        # split time and command and comment
        tima = "_".join(j[:5])
        txta = " ".join(j[5:]).split("#")[0]
        # clean long commands
        txta = txta.replace("DISPLAY=:0 DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/1000/bus", "")
        # this is not going to be used anymore
        txta = txta.replace("/usr/bin/python3", "").strip(" ")
        txtb = " ".join(j[5:]).split("#")[-1]
        # print
        tag = fg.default
        if running is not None and txta.split()[2] in running:
            tag = fg.green
        print(f" {tag}{tima:16s} {txta:50s} # {txtb} ", fg.default)



#=========================================================================
#
#-------------------------------------------------------------------------

def just_showlist_cron():
    """
    aas it says
    """
    env = os.environ.copy()
    CMD = "crontab -l"
    args = shlex.split(CMD)
    ret = sp.check_output(args)#Popen(args, env=env, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
    ret = ret.decode("utf8").strip().split("\n")
    ret = [ x.strip() for x in ret if str(x).find("#") != 0 ]
    #for i in ret:
    #    #process.poll()
    #    print(i)
    crfile = f"/tmp/crontablinst_{config.get_current_user()}"
    with open(crfile, 'w') as f:
        f.write("\n".join(ret))
    CMD = f"cat {crfile}; read"
    if libs_screen.is_session_desktop():
        args = shlex.split(f"xterm -e bash -c '{CMD}'")
        process = sp.Popen(args,  stdout=sp.DEVNULL, stderr=sp.DEVNULL)
        process.poll()
    else:
        args = shlex.split(f"bash -c '{CMD}'")
        res = sp.call(args)



#=========================================================================
#
#-------------------------------------------------------------------------


def remove_fullpath(fullpath, item):
    """
    leave just lastpath/tag
    """
    return item.replace(fullpath + '/', '', 1)

#=========================================================================
#
#-------------------------------------------------------------------------



def get_comment(tag):
    if len(config.ALL_COMMENTS) == 0:
        find_second_level_executables()
    if tag in config.ALL_COMMENTS:
        return config.ALL_COMMENTS[tag]
    else:
        return None


#=========================================================================
#
#-------------------------------------------------------------------------


def fill_current_DTS():
    """
    extract time interval from cron and fill DICT
    use the real installation site of this code todetect!
    """
    #global config.INSIDE_DTS
    mycron.read()
    #config.debug_write(f"Warning:   cron is read len=={len(mycron)} ", "LC")

    file_path = os.path.abspath(sys.modules['__main__'].__file__)
    for job in mycron:
        tag2 = str(job).split(file_path)[-1].strip().split(" ")[1].strip()
        #print( f"{tag2:13s}", ">>>", job.slices)
        config.INSIDE_DTS[tag2] = job.slices
        #if tag2.find("flash") >= 0:
        #    config.debug_write(f"Warning:  flash {config.INSIDE_DTS[tag2]} ", "LC")
    #print(dt.datetime.now())

#=========================================================================
#
#-------------------------------------------------------------------------


def get_DT(tag):
    """
    extract time interval from cron
    """
    #if len(config.INSIDE_DTS) == 0:
    #fill_current_DTS()
    #global config.INSIDE_DTS
    if not tag in config.INSIDE_DTS:
        print(f"X... no cron data for this site-code")
        print(f"X... unknown tag {tag} for DT column:", config.INSIDE_DTS.keys())
        return None
    DT = str(config.INSIDE_DTS[tag]).strip().split(" ")
    res = ""
    if DT[0] == "*":
        pass #res = f"{res} 0m"
    else:
        res = f"{res} {DT[0]}m"

    if DT[1] == "*":
        pass #res = f"{res} 0h"
    else:
        res = f"{res} {DT[1]}h"

    if DT[2] == "*":
        pass #res = f"{res} 0d"
    else:
        res = f"{res} {DT[2]}d"

    res = res.replace("*", "").strip()
    if len(res) == 0:
        res = "1m"
    #if tag.find("flash") >= 0:
    #    config.debug_write(f"info...:  flash {config.INSIDE_DTS[tag]} # {res}", "LC")
    return res


#=========================================================================
#
#-------------------------------------------------------------------------


def get_fullpath(tag):
    if len(config.ALL_PATHS) == 0:
        find_second_level_executables()
    if tag in config.ALL_PATHS:
        return config.ALL_PATHS[tag]
    else:
        return None


#=========================================================================
#
#-------------------------------------------------------------------------


def extract_one_comment(file_path):
    #print("D... opening", file_path)
    with open(file_path) as f:
        shebang = f.readline().strip()
        #print("D... shebang == ", shebang)
        if shebang.find("#") == 0:
            comment = f.readline().strip()
            #print("D... comment == ", comment)
            if comment.find("#") == 0:
                return comment
            else:
                return None
        else:
            return None


#=========================================================================
#
#-------------------------------------------------------------------------


def find_second_level_executables():
    """
    parse all execs in second level of directory
    """
    directory = config.CONFIG['services']
    directory = os.path.expanduser(directory)

    config.debug_write('parsing SECOND.LEVEL=services', "CR")
    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... {dt.datetime.now()} ... in find_second_level ... services is @ {directory} \n")

    if not os.path.exists(directory):
        print("X... configuration of services' directory is incorrect")
        print("X... BAD DIRECTORY", directory)
        print("X... SEE CONFIG AT", config.CONFIG['filename'] )
        sys.exit(1)

    executables = []
    for subdir in os.listdir(directory):
        subdir_path = os.path.join(directory, subdir)
        if os.path.isdir(subdir_path):
            for file in os.listdir(subdir_path):
                file_path = os.path.join(subdir_path, file)
                if os.path.isfile(file_path) and os.access(file_path, os.X_OK):
                    executables.append(file_path)
                    tag = file_path.split("/")[-1] # TAG
                    COM = extract_one_comment(file_path)
                    config.ALL_COMMENTS[tag] = COM
                    config.ALL_PATHS[tag] = file_path
    return executables

#=========================================================================
#
#-------------------------------------------------------------------------

def list_services2():
    """
    return  just folder/exec LIST
    """
    exelist = find_second_level_executables()
    fullp = os.path.expanduser(config.CONFIG['services'])
    #print(exelist)
    cut = []
    for i in exelist:
        cut.append( remove_fullpath(fullp, i) )
    return cut

#=========================================================================
#
#-------------------------------------------------------------------------

def list_services1():
    """
    return just exec LIST
    """
    cut = list_services2()
    cut = [ i.split("/")[-1] for i in cut]
    duplicates = [item for item in set(cut) if cut.count(item) > 1]
    if len(duplicates) > 0:
        print(bg.red, fg.white, "X... duplicate filenames: ", duplicates, fg.default, bg.default)
    return cut


#=========================================================================
#
#-------------------------------------------------------------------------


def is_in_allowed(tag):
    """
    is the tag in allowed-list ?
    """
    if tag in list_services1():
        return True
    else:
        return False

#=========================================================================
#
#-------------------------------------------------------------------------
#

def get_now():
    res = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
    return res


#=========================================================================
#
#-------------------------------------------------------------------------
#

def list_crons( ):
    """
    just list crons that contain MYENV - the last word is the REALNAME
    """
    global mycron
    alist = []
    for job in mycron:
        sjob = str(job)
        #print(type(sjob))
        if sjob.find(config.MYENV) > 0:
            name = sjob.split(config.MYENV)[-1]
            if "#" in name:
                name = name.split("#")[0].strip()
            name = name.split(" ")[-1]
            #print(sjob, name)
            alist.append(name)
    return alist
#=========================================================================
#
#-------------------------------------------------------------------------
#

def allocate( exe):
    """
    use which , find location of exe, return complete path
    """
    config.debug_write(f"ALLOCATE ... the exe is {exe}", "CR")

    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... in ALLOCATE ... the exe is {exe} \n")

    if os.path.exists( exe): # This is total path
        #with open("/tmp/cronvice.log", "a") as f:
        #    f.write(f"i... ret  ... the exe is {exe} \n")
        return exe

    # try to find it ith which
    program_path = shutil.which( exe )
    config.debug_write(f"shutil result is {program_path}", "CR")

    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... ret  ... the shutil result is {program_path} \n")

    # try some other locations
    if program_path is None:
        config.debug_write(f"Progpath NONE", "CR")
        #with open("/tmp/cronvice.log", "a") as f:
        #    f.write(f"i... ppath was None  \n")
        home_dir = os.environ.get('HOME')
        #with open("/tmp/cronvice.log", "a") as f:
        #    f.write(f"i... return  ... 1a homedir: {home_dir} \n")
        file_path = os.path.join(home_dir, '.local', 'bin', exe)
        #with open("/tmp/cronvice.log", "a") as f:
        #    f.write(f"i... return  ... 1a  joined {home_dir} - {file_path} \n")
        if os.path.exists(file_path):
            #with open("/tmp/cronvice.log", "a") as f:
            #    f.write(f"i... return  ... 1 {file_path} \n")
            config.debug_write(f"ret {file_path}", "CR")

            return file_path

    # try some other locations
    if program_path is None:
        home_dir = os.environ.get('HOME')
        file_path = os.path.join(home_dir, 'bin', exe)
        if os.path.exists(file_path):
            config.debug_write(f"ret2 {file_path}", "CR")

            #with open("/tmp/cronvice.log", "a") as f:
            #    f.write(f"i... return  ... 2 {file_path} \n")
            return file_path

    return program_path # wil do None eventually


#=========================================================================
#
#-------------------------------------------------------------------------
#

def view_cron( short=False):
    global mycron
    SCRDMS = f"/usr/bin/screen -dmS " #SPACE IS IMPORANT
    sessions = libs_screen.list_screen_sessions() # PREPARE
    #print(sessions)
    for job in mycron:
        txt = job.command
        if txt[0] == "#": continue
        if short:
            if txt.find(SCRDMS) > 0:
                ATAG = txt.split(SCRDMS )[1].split()[0]
                BCMD = " ".join(txt.split(SCRDMS )[1].split()[1:])

                IIS = is_in_screen(ATAG, sessions)
                if IIS:
                    col = f"{fg.green}* "
                else:
                    col = f"{fg.red}x "
                print( f" {col}{ATAG:12s}{fg.default} - {BCMD}" )
            #else:
            #    print(" - ")
        else:
            print(txt)


#=========================================================================
#
#-------------------------------------------------------------------------
#

def is_in_cron(cron, tag):
    """
    Not usefull with  cronvice in  crontab
    """

    #file_path = os.path.abspath(__file__)
    file_path = os.path.abspath(sys.modules['__main__'].__file__)

    RMTG = f"{file_path}"
    for job in cron:
        if job.command.find(RMTG) > 0 and job.command.endswith(f" r {tag}") > 0: # no SPACE
            return True
    return False


#=========================================================================
#
#-------------------------------------------------------------------------
#

def del_job(  tag): #REAL
    """
    delete with the screen
    """
    ACT = False
    #file_path = os.path.abspath(__file__)
    file_path = os.path.abspath(sys.modules['__main__'].__file__)

    # RMTG = f"screen -dmS {tag} " #SPACE IMPORTANT
    RMTG = f"{file_path}"
    for job in config.mycron:
        #print(f"/{job.command}/..")
        if job.command.find(RMTG) > 0 and job.command.endswith(f" r {tag}") > 0: # no space
            print(f"i... removing /{RMTG}/{tag}/ ")#... {job}")
            config.mycron.remove(job)
            ACT = True
    if ACT:
        config.mycron.write()
    else:
        print("X... nothing deleted")



#=========================================================================
#
#-------------------------------------------------------------------------
#

# def add_job_anycommand( cron, tag):
#     """
#     first attempt to add any command, basically
#     """

#     if is_in_cron(cron, tag):
#         print(f"X... already present {tag}")
#         return
#     ENV = config.MYENV #"DISPLAY=:0 DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/1000/bus"
#     SCR = f"/usr/bin/screen -dmS {tag} " #SPACE IS IMPORANT
#     #make code sequence with &&
#     #CODE = f"echo i...running {tag} && /home/ojr/.local/bin/notifator b hello_{tag} && echo i...running {tag} && sleep 30"
#     MAINCODE = 'notifator'
#     MAINCODE = allocate(MAINCODE)
#     CODE = f"{MAINCODE} b hello_{tag}_{get_now()} && sleep 30"
#     # encapsulate to ''
#     CODE = f"bash -c '{CODE}'"
#     CMD = f"{ENV} {SCR} {CODE}"


#     print("i... adding job", tag)

#     job = cron.new(command=CMD)
#     job.minute.every(1)
#     #job.set_command("new_script.sh")
#     job.set_comment("test of notifator")
#     cron.write()




#=========================================================================
#
#-------------------------------------------------------------------------
#


def is_int(n):
    if str(n).find(".")>=0:  return False
    if n is None:return False
    try:
        float_n = float(n)
        int_n = int(float_n)
    except ValueError:
        return False
    else:
        return float_n == int_n



def add_job(  tag, every="1m"): #REAL
    """
    cron:  * * * * * display dbus python3 path/to/cronvice!!! r  tag
    """
    if is_in_cron(config.mycron, tag):
        print(f"X... already present {tag}")
        return

    if not is_in_allowed(tag):
        print(f"X... this job is not allowed {tag}")
        return


    ENV = config.MYENV #"DISPLAY=:0 DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/1000/bus"

    #file_path = os.path.abspath(__file__)
    file_path = os.path.abspath(sys.modules['__main__'].__file__)

    #return

    #SCR = f"/usr/bin/screen -dmS {tag} " #SPACE IS IMPORANT
    ###make code sequence with &&

    #MAINCODE = 'notifator'
    #MAINCODE = allocate(MAINCODE)
    MAINCODE = file_path
    INTERPRETTER = config.CONFIG['interpretter']
    # with uv not this ................ ------------------------ VENV
    CODE = ""
    #INTERPRETTER = "~/.venv/myservicenv/bin/python3"
    INTERPRETTER = os.path.expanduser(INTERPRETTER)
    if os.path.exists(INTERPRETTER):
        CODE = f"{INTERPRETTER} {MAINCODE} r {tag}"
    else:
        CODE = f"{MAINCODE} r {tag}"
    # ************************* doesnt work, I must activate the envirohnment ****
    # ************************* doesnt work, I must activate the envirohnment ****
    # ************************* doesnt work, I must activate the envirohnment ****
    CODE = f"{MAINCODE} r {tag}"
    # ----------------------------------------------------------
    #### encapsulate to ''
    #CODE = f"bash -c '{CODE}'"
    #CMD = f"{ENV} {SCR} {CODE}"
    CMD = f"{ENV} {CODE}"

    print("i... adding job", tag)

    job =config.mycron.new(command=CMD)

    length = "".join(every[0:-1])
    print(f"D...   int==/{length}/  char==/{every[-1]}/")
    if is_int(length):
        length = int(length)
    else:
        length = 1


    if every[-1] == "m":
        print("i... minute")
        job.minute.every(length)
    elif every[-1] == "h":
        print("i... hour")
        if length == 1:
            job.minute.every(59)
        else:
            job.hour.every(length)
    elif every[-1] == "d":
        if length == 1:
            print("i... day")
            job.hour.every(23)
            job.minute.every(59)
        else:
            job.day.every(length)
    else:
        print("X... unknown time unit for cron /m-h-d/ :", every)
        return
    #job.set_command("new_script.sh")
    job.set_comment("cronvice job")
    config.mycron.write()


#=========================================================================
#
#-------------------------------------------------------------------------
#

def see_job(  tag):
    """
    xxxxxxxxx empty check NOT USED!!!!!!!!!!!!!!!!
    """
    if is_in_cron(config.mycron, tag):
        print(f"X... already present {tag}")
        return
    if not is_in_allowed(tag):
        print(f"X... this job is not allowed {tag}")
        return
    ENV = config.MYENV #"DISPLAY=:0 DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/1000/bus"

    #file_path = os.path.abspath(__file__)
    file_path = os.path.abspath(sys.modules['__main__'].__file__)

    MAINCODE = file_path
    #INTERPRETTER = config.INTERPRETTER

    INTERPRETTER = config.CONFIG['interpretter']
    # with uv not this ................ ------------------------ VENV
    #INTERPRETTER = "~/.venv/myservicenv/bin/python3"
    INTERPRETTER = os.path.expanduser(INTERPRETTER)
    if os.path.exists(INTERPRETTER):
        CODE = f"{INTERPRETTER} {MAINCODE} r {tag}"
    else:
        CODE = f"{MAINCODE} r {tag}"
    ####### encapsulate to ''
    ###CODE = f"bash -c '{CODE}'"
    ###CMD = f"{ENV} {SCR} {CODE}"
    #CMD = f"{ENV} {CODE}"

    #print("i... adding job", tag)
    print(CMD)
    #job =config.mycron.new(command=CMD)
    #job.minute.every(1)
    ####job.set_command("new_script.sh")
    #job.set_comment("test of notifator")
    #config.mycron.write()




#=========================================================================
#
#-------------------------------------------------------------------------
#
########################################################################

def start(tag, duotag): #REAL   is calledby run_job
    """
    starts screen in BG with environment...... ;  Screen Tag Added
    """
    config.debug_write(f'start TAG="{tag}" -------------------> "START"', "CR")

    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... in START ... the tag is {tag} \n")
    #ENV = "DISPLAY=:0 DBUS_SESSION_BUS_ADDRESS=unix:path=/run/user/1000/bus"
    ENV = ""
    # Set environment variables
    env = os.environ.copy()
    env["DISPLAY"] = ":0"
    env["DBUS_SESSION_BUS_ADDRESS"] = "unix:path=/run/user/1000/bus"
    # ---- telegrf uses cruditi!!!
    full_path = os.path.expanduser("~/.local/bin")
    env["PATH"] = env.get("PATH", "") + ":" + full_path

    SCR = f"/usr/bin/screen -dmS {tag}{config.CONFIG['screentag']} " #SPACE IS IMPORANT
    ################################## ENV SCR and CODE ###############################
    #make code sequence with &&

    #MAINCODE = 'notifator'
    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... in START ... the main is {MAINCODE} \n")
    #
    #MAINCODE = allocate(MAINCODE)

    MAINCODE = f"{os.path.expanduser(config.CONFIG['services'])}/{duotag}"
    # putting EXEC in
    #####MAINCODE = f"{os.path.expanduser(config.CONFIG['services'])}/{duotag}"
    EXECUDIR = MAINCODE.split("/")[:-1]
    EXECUDIR = "/".join(EXECUDIR)
    config.debug_write(f"start % {MAINCODE} %", "CR")

    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... in START ...   {MAINCODE} \n")
    CODE = f" cd {EXECUDIR} && exec {MAINCODE}  && sleep 1"  # exec should replace bash and behave better
    #CODE = f"{MAINCODE} b hello_{tag}_{get_now()} && sleep 30"
    #####################################################################################
    # encapsulate to ''

    CODE = f"bash -c '{CODE}'" # REPLACED 20250625 NONONO
    #NONONO CODE = f"exec '{CODE}'" # this should replace shell and sojhuld solve the ppid 1 adter killing screen

    CMD = f"{ENV} {SCR} {CODE}"
    args = shlex.split(CMD)
    #print(args)
    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... in START ...and... the args is {args} \n")
    config.debug_write(f"{args}", "CR")
    #config.INSIDE_LAST_RUN[tag] = dt.datetime.now()
    # -------- in case of telegraf:
    process = sp.Popen(args, env=env, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
    #process = sp.Popen(args, env=env, stdout=sp.DEVNULL, stderr=sp.DEVNULL, close_fds=True)
    #process.detach = True
    process.poll()
    config.PROC_OBJ[tag] = {}
    config.PROC_OBJ[tag]['obj'] = process
    config.PROC_OBJ[tag]['time'] = dt.datetime.now()
    config.PROC_OBJ[tag]['dt'] = None
    config.debug_write(f"LAUNCHED {config.PROC_OBJ[tag]} ErrCode={process.poll()} ######", "CR")
    #config.debug_write(f"DETACHED {config.PROC_OBJ[tag]}  ######", "CR")

#=========================================================================
#
#-------------------------------------------------------------------------

def run_job( tag): # REAL CALLS START!!!
    """
    if it is in cron and screen not running......
    """
    config.debug_write(f'run TAG=="{tag}"    ---------------------->run_job?', "CR")
    #with open("/tmp/cronvice.log", "a") as f:
    #    f.write(f"i... in run_job ...and... the tag is {tag} \n")


    # if I test first if it is running, no need to lookup secondlevelservices
    #config.debug_write(f"Warning: 1", "CR")
    sessions = libs_screen.list_screen_sessions() # PREPARE but NOOrun secondslevelservices too
    #print("i... in runjob")
    #config.debug_write(f"Warning: 2", "CR")
    if libs_screen.is_in_screen(tag, sessions):
        #config.debug_write(f"Warning: 3", "CR")

        config.debug_write(f"................................. already running", "CR")
        #print(f"X...  {fg.yellow} already running {tag} {fg.default} ")
        return
    else:
        print(f"X...  {fg.green} {tag} is NOT running now ... {fg.default} ")


    if is_in_allowed(tag): # this calls secondlevelserices
        #with open("/tmp/cronvice.log", "a") as f:
        #    f.write(f"i... in run_job ...and... # sessions is {len(sessions)} \n")
        config.debug_write(f"run GO for start", "CR")
        print(f"i... {fg.orange} Starting {tag} .... {fg.default}" )
        li2 = list_services2()
        #print(li2)
        for item in li2:
            folder, executable = item.split('/')
            if executable == tag:
                print("D...  folder/item/tag", folder, item, tag)
                start(tag, item)


#=========================================================================
#
#-------------------------------------------------------------------------
#

def mod_job(cron, tag):
    del_job(cron, tag)
    add_job(cron, tag)

if __name__ == "__main__":
    Fire({"a": add_job,
          "r": run_job,
          "d": del_job
          })



#!/usr/bin/env python3

# to override print <= can be a big problem with exceptions
# from __future__ import print_function # must be 1st
# import builtins

# import sys

from fire import Fire

# from cronvice.version import __version__
# from cronvice import unitname
from cronvice import config
from cronvice import libs_screen
from cronvice import libs_cron
# import list_screen_sessions
# import time
# import datetime as dt
from console import fg, bg
import time
# import os

import pandas as pd
import numpy as np
from terminaltables import SingleTable
import datetime as dt
from dateutil import parser
#import dateutil
# =============================================================
#
#--------------------------------------------------------------

def decode_datetime(date_string):
    return parser.parse(date_string, dayfirst=True)

def crtable():
    """
    READ FROM THE : screens   crontab and DIRECTORY
    """
    NOBAR = "----"
    scrlist = libs_screen.list_screen_sessions()
    crolist = libs_cron.list_crons()
    #print(scrlist)
    #time.sleep(3)
    df = pd.DataFrame(  columns=['cron', 'screen',  'elapsed', 'comment', 'DT', 'path'])
    #print(df)
    ix = 0
    # list here all what is in cron
    for i in crolist:
        name = i #item[0].split(".")[1]
        comment = libs_cron.get_comment(name)
        libs_cron.fill_current_DTS() # refresh all DT - time consuming MUST COME BEFORE DT
        dtslice = libs_cron.get_DT(name)
        #config.debug_write(dtslice, "DF")
        if not comment is None: # ------ comment on 2nd line ---------
            comment = comment.strip().replace("#", "", 1) # 1st occurence
            comment = comment.strip().replace("myservice_description:", "").strip()
            #comment = comment.split("#")[0].strip() # not

        else:
            comment = "..."
        df.loc[ ix, 'cron'] = name
        df.loc[ ix, 'screen'] = NOBAR
        #df.loc[ ix, 'time'] = NOBAR

        # if it was run
        interval = NOBAR

        ### mystery  ***** NOT REALLY * running is always from outside; here there is no idea
        # NEVERHAPPENS
        if name in config.PROC_OBJ.keys():
            #config.debug_write(f"Warning:  key {name} in obj", "DF")
            if 'dt' in config.PROC_OBJ[name]:
                interval = config.PROC_OBJ[name]['dt']
                #pass
        #     with open("/tmp/cronvice.log", "a") as f:
        #         f.write(f"i... in DF ... {name} at {config.INSIDE_PROC_OBJ[name]}\n")

        #     if type(config.INSIDE_LAST_RUN[name]) == dt.timedelta:
        #         interval = round(config.INSIDE_LAST_RUN[name].total_seconds(), 1)
        #         with open("/tmp/cronvice.log", "a") as f:
        #             f.write(f"i... in DF ... {name} timedelta {config.INSIDE_LAST_RUN[name]}\n")

        df.loc[ ix, 'elapsed'] = interval #
        df.loc[ ix, 'comment'] = comment.format(fg=fg, bg=bg)
        df.loc[ ix, 'DT'] = dtslice
        # crashes during changes
        df.loc[ ix, 'path'] = "inknown"
        #
        try:
            dftempo = libs_cron.get_fullpath(name).split("/")
            df.loc[ ix, 'path'] = libs_cron.get_fullpath(name).split("/")[-2]
        except:
            pass

        #df.loc[ ix, 'screen'] = NOBAR # -AGAIN !!!!
        cgr = [] # APRIORI PUT NEGATIVE
        if df.loc[ ix, 'comment'].find("#") > 0:
            cgr = df.loc[ ix, 'comment'].split("#")
            #df.loc[ index, 'comment'] = cgr[0] # DO NOT REPAIR COMMENT here
            #if len(cgr) > 1 and name != NOBAR:
            #    df.loc[ index, 'screen'] =  cgr[1].strip()
            if len(cgr) > 2:
                df.loc[ ix, 'screen'] =  cgr[2].strip()


        #if name.find("flash") >= 0:
        #    config.debug_write(f"{dtslice}, {ix}, {df.loc[ ix, 'DT']}", "XX")
        ##df.loc[ 0, 'time'] = ctime
        ix += 1

    #print(df)
    #df.set_index( df['cron'], inplace=True)
    #print("-------------", list(df['cron']), name)
    #time.sleep(10)
    # ---------------------------- SCREENS  -------------------------
    if (scrlist is None) or (len(scrlist) < 1):
        pass#return df
    # ----------------------------------------- RUN THROUGH SCREENS -------------------------------
    if scrlist is not None:
        for i in scrlist:
            item = i.strip().split("\t")
            if len(item) < 2:
                config.debug_write(f"Error: in screen ls: {i} {scrlist}", "DF")
                config.debug_write(f"Error: in screen", "DF")
                return df # HARDER !!!!!
                continue # BUG WITH SCREEN LS - myservice was failing a lot
            ctime = item[1].strip("()")
            name = item[0].split(".")[1] # THIS WILL BE WITH _CX
            if name.find( config.CONFIG['screentag'] ) > 0:
                #print(f"{line[0]:20s} -  {xtime}  -  {xlong}")
                name = name.split( config.CONFIG['screentag'] )[0]
            else:
                name = None


            #print(name, type(name))
            if name in list(df['cron']):  # ----- SCREEN IS RUNNING -------------------------------------
                index = df.index[df['cron'] == name][0] # get index of the row where col:cron==name
                df.loc[ index, 'screen'] = name
                cgr = []
                if df.loc[ index, 'comment'].find("#") > 0:
                    cgr = df.loc[ index, 'comment'].split("#")
                    # df.loc[ index, 'comment'] = cgr[0] # do not repair comment here
                    if len(cgr) > 1 and name != NOBAR:
                        df.loc[ index, 'screen'] =  cgr[1].strip()
                    #if len(cgr) > 2 and name == NOBAR: # this doesnt happen
                    #    df.loc[ index, 'screen'] =  cgr[2].strip()
                #if
                #df.loc[ 0, 'screen'] = name
                #df.loc[ index, 'time'] = ctime
                #dateutil.parser.parserinfo(dayfirst=True)
                date_time = decode_datetime(ctime)
                #date_time = dt.datetime.strptime(ctime, "%d/%m/%y %H:%M:%S")
                time_difference = dt.datetime.now() - date_time
                xe = f"{time_difference.days}d {time_difference.seconds // 3600}h {(time_difference.seconds % 3600) // 60}m"
                df.loc[ index, 'elapsed'] = xe
                pass
            else: # ----- SCREEN IS NOT RUNNING -------------------------------------------------------------------
                pass
            #ix += 1
        #time.sleep(10)


    # ---------------- repair comments
    ix = 0
    for i in crolist:
        if df.loc[ ix, 'comment'].find("#") > 0:
            cgr = df.loc[ ix, 'comment'].split("#")
            df.loc[ ix, 'comment'] = cgr[0] # repair comment
        ix += 1
    return df

# =============================================================
#
#--------------------------------------------------------------

def create_dummy_df():
    """
    create dummy dataframe from scratch.... test purposes
    """
    columns = ["a", "b", "c", "_fg", "_bg"]
    # columns1=[x for x in columns if x[0]!="_"]
    df = pd.DataFrame(np.random.randint(0, 9, size=(11, len(columns))),
                      columns=columns)
    df["_fg"] = fg.lightgray  # fg.default
    df["_bg"] = bg.default

    # --------------------------- default pattern ------------
    for i, row in df.iterrows():
        if i % 3 == 0:
            df.loc[i, ["_bg"]] = bg.darkslategray  # bg.dimgray#bg.darkslategray
        else:
            df.loc[i, ["_bg"]] = bg.default  # bg.black

        #if i % 5 == 0:
        #    df.loc[i, ["_fg"]] = fg.lightgreen  # lightyellow

    return df


def inc_dummy_df(df):
    """
    increase df cells by unit for the demo dummy table
    """
    for i, row in df.iterrows():
        df.iloc[i, :-2] = df.iloc[i, :-2] + 1  # loc doesnt work, iloc is ok
    return df



def enhance_df(df):
    """
    With a read dataframe, you need to enhance with _bg and _fg to be displayable
    """
    #columns = ["a", "b", "c", "_fg", "_bg"]
    # columns1=[x for x in columns if x[0]!="_"]
    #df = pd.DataFrame(np.random.randint(0, 9, size=(11, len(columns))),
    #                  columns=columns)
    df["_fg"] = fg.lightgray  # fg.default
    df["_bg"] = bg.default

    # --------------------------- default pattern ------------
    for i, row in df.iterrows():
        if i % 3 == 0:
            df.loc[i, ["_bg"]] = bg.darkslategray  # bg.dimgray#bg.darkslategray
        else:
            df.loc[i, ["_bg"]] = bg.default  # bg.black
        #
        #if i % 5 == 0:
        #    df.loc[i, ["_fg"]] = fg.lightgreen  # lightyellow
    return df




# ======================================================
def show_table(df, selection="3", return_subdf=False):
    """
    enhance the df and display fancy. Also return df when selection
    """

    row_n = [
        "1",
        "2",
        "3",
        "4",
        "5",
        "6",
        "7",
        "8",
        "9",
        "a",
        "b",
        "c",
        "d",
        "e",
        "f",
        "g",
        "h",
        "i",
        "j",
        "k",
        "l",
        "m",
        "n",
        "o",
        "p",
        "q",
        "r",
        "s",
        "t",
        "u",
        "v",
        "w",
        "x",
        "y",
        "z",
    ]
    #---------------

    if return_subdf:
        # Convert selection to a list of integers (0-based index)
        # selected_indices = [ord(char) - 1 for char in str(selection) ]
        selected_indices = [row_n.index(char) for char in selection]
        selected_indices = [i for i in selected_indices if i < len(df)] # avoid crasjh
        return  df.iloc[selected_indices]

    dfenha = df.copy()
    dfenha = enhance_df(dfenha)
    #-
    dfpure = df.copy()
    # dfpure.drop(columns=["_fg", "_bg"], inplace=True) # oooo

    rows = dfpure.values.tolist() #-------------------->>>>>
    rows = [[str(el) for el in row] for row in rows] #->>>>>
    # columns = df.columns.tolist()
    #
    columns2 = [x for x in list(dfenha.columns) if x[0] != "_"]
    #
    tab_header = [["n"] + columns2]  #----------
    #
    # tab_header = [  f"{fg.white}{x}{fg.default}" for x in tab_header] # NOTW
    # data = [['a','b'], ['ca','cb']]
    #========================= start to construct the table ============
    tab_src = tab_header.copy()

    # nn=0  #I use index
    padding = "  "  # nicer bars
    for index, row in dfenha.iterrows():
        # i take row from pure
        row = list(dfpure.loc[index, :])
        fgcol = fg.white
        fgcol = dfenha.loc[index, ["_fg"]].iloc[0]
        bgcol = dfenha.loc[index, ["_bg"]].iloc[0]
        if selection is not None and row_n[index] in list(selection):
            # print(index, selection)
            bgcol = bg.yellow4  # df.loc[index,['_fg']][0]

        # print(bgcol)
        # print(index, row ) # list of pure df cols for row
        row = [row_n[index]] + row

        for j in range(len(row)):  # change color for all columns
            row[j] = (
                fgcol
                + bgcol
                + padding
                + str(row[j])
                + padding
                + bg.default
                + fg.default
            )



        tab_src.append(row)  # prepend to list /THE TABLE TO DISPLAY/
        # nn+=1

    # ==================== HERE IS THE OBJECT=========
    table = SingleTable(tab_src)
    table.padding_left = 0
    table.padding_right = 0
    # blessings terminal() t.clear()

    # --------- if too wide - i think
    if not table.ok:
        table.padding_left = 0
    if not table.ok:
        table.padding_right = 0
    while not table.ok:
        # if bad size
        # remove columns here
        j = 0
        for k in tab_src:
            tab_src[j] = k[:-1]
            j+=1
        table = SingleTable(tab_src)
        table.padding_left = 0
        table.padding_right = 0

    print(table.table)





def main():
    """
    Show table
    """
    df = create_dummy_df()
    df = inc_dummy_df(df)
    move_cursor(15, 4)
    show_table(df) # Testing in main()


if __name__ == "__main__":
    Fire(main)



#!/usr/bin/env python3
"""
  THe goal is to have a general interpretter that:
   - makes some shell actions
   - performs some operations on objects OR file
"""
#
#  I want to use cmd_parser and call the functions with parameters BY "string cmdline parame"" like Fire()
#
#
from fire import Fire
import sys
from cronvice.version import __version__
from cronvice import config
from cronvice.config import  move_cursor
#from cronvice.config import  DEBUG
#
from cronvice import cmd_parser
import os
#
#
#
import subprocess as sp
from console import fg,bg
import glob
import shlex
import re


# =========  all commands are in 2 main groups ===========-----------------
#  1      shell (just pass to shell)
#  2      local - each one needs to have its own module defined  fn_*.py
#-------------------------------------
# ---------------LOCAL COMMANDS ARE IN 2 MAIN GROUPS ==========
#  2a     argument is none OR a FILE
#  2b     argument is an object
# -------------------------------------------------------------
# SHELL COMMANDS with files as parameters (ll is extended here)
KNOWN_COMMANDS_SHELL_FS = ['ls','ll','fdfind','head','tail','cat']
# other shell commands
KNOWN_COMMANDS_SHELL_OTHER = ['ag']
# call functions@fn_modules -  objects as parameters -  must be defined
# KNOWN_COMMANDS_LOCAL = ['lo','show','zoom','unzoom','connect','reset']
KNOWN_COMMANDS_LOCAL_OTHER = ['reset']
#  content of FUNCTIONs -------- fn_.py in the project
#  load  - FS
#  lo
#  print  - OBJ
#  show - OBJ
KNOWN_COMMANDS_LOCAL_OBJ = []# ['show']
# call functions@fn_modules - files as parameters
KNOWN_COMMANDS_LOCAL_FS = []# ['load']
#


def find_known_commands_type(file_path):
    """
    check the fn_py file and find out which type it is  OBJ or FS
    """
    pattern = re.compile(r'^\s*KNOWN_COMMANDS_LOCAL_TYPE\s*=\s*"(FS|OBJ|OTHER)"')
    #print("D...  checking KCLT @ ", file_path)
    with open(file_path, 'r') as file:
        for line in file:
            #if line.find("KNOWN_COMMANDS")>=0: print(" ...       ",line.strip())
            match = pattern.match(line)
            if match:
                return match.group(1) #line.strip()
    return None



# ************************************************************************ running at start!!!!!!!!!
# ************************************************************************ running at start!!!!!!!!!
# ************************************************************************ running at start!!!!!!!!!
# ************************************************************************ running at start!!!!!!!!!
def init_interpretter():
    # ---- good for completions AND ALSO NEW FUNCTIONS!!! in ACTUAL FOLDER
    #---this doesnt work
    if config.DEBUG:
        print("____________________________________________________________________interpretter")
    allfiles = glob.glob("*")

    # ---- AVAILABLE FUCTIONS ----------== COMMANDS == maybe ONLY ONCE PER RUN
    # --- a trick, I look for other local fn_py
    all_pyfiles = cmd_parser.listpy( also_local=True, DEBUG=False)
    for i in all_pyfiles:
        #print("D...   all_pyfiles:",i)
        # if i[:3] == "fn_" and i[-3:]==".py": # Prefix is already defined
        #print("D...  looking for functions:",i)
            #print(i[-3:]==".py")
        name = os.path.basename(i)
        #print(name)
        name = name.lstrip("fn_")
        name = name.rstrip(".py")
        #print(f"A... adding {fg.yellow} {name} {fg.default}")
        res = find_known_commands_type(i)
        if res is None:
            print(f"X... {fg.red} 'KNOWN_COMMANDS_LOCAL_TYPE = ' is missing in {i} {fg.default} ")
            continue
        elif res =="FS":
            KNOWN_COMMANDS_LOCAL_FS.append(name)
        elif res =="OBJ":
            KNOWN_COMMANDS_LOCAL_OBJ.append(name)
        elif res =="OTHER":
            KNOWN_COMMANDS_LOCAL_OTHER.append(name)
        else:
            print(f"X... {fg.red} PROLBEM???  'KNOWN_COMMANDS_LOCAL_TYPE = ' @ {i} {fg.default} ", res)

    if config.DEBUG:
        print("i... Object commands:",KNOWN_COMMANDS_LOCAL_OBJ)
        print("i... File   commands:",KNOWN_COMMANDS_LOCAL_FS)
        print("i... Other  commands:",KNOWN_COMMANDS_LOCAL_OTHER)

     # --------------------------------------------------------
    # update AVAILABLE -  FILES and OBJECTS.
    # --------------------------------------------------------
    #============================================= prepare for completions =======================
    # ============================================================================= summary
    KNOWN_COMMANDS = KNOWN_COMMANDS_SHELL_FS + \
        KNOWN_COMMANDS_SHELL_OTHER + \
        KNOWN_COMMANDS_LOCAL_FS +\
        KNOWN_COMMANDS_LOCAL_OTHER + \
        KNOWN_COMMANDS_LOCAL_OBJ


    # now create nested completion dict  EVERY COMMAND HAS AUTOCOMPLETE
    KNOWN_COMMANDS_DICT = {}
    for i in KNOWN_COMMANDS:
        KNOWN_COMMANDS_DICT[i]=None

    # now create special completion for filemanagement
    #   ??this is created only once when here....need to be moved
    for i in KNOWN_COMMANDS_SHELL_FS+KNOWN_COMMANDS_LOCAL_FS:
        KNOWN_COMMANDS_DICT[i] = {}
        for j in allfiles:
            KNOWN_COMMANDS_DICT[i][j] = None

    #
    # completely useless here, but I keep it for the symetry and keyboard_remote_start
    #
    allobjects =  [] #['obj1']
    # now create special completion for memory OBJECTS
    for i in KNOWN_COMMANDS_LOCAL_OBJ:
        KNOWN_COMMANDS_DICT[i] = {}
        for j in allobjects:
            KNOWN_COMMANDS_DICT[i][j] = None
    #========================================================== completions created...............

    #print(KNOWN_COMMANDS_DICT)



def respond(inp):
    """
    orphaned function.....
    """
    done = False
    while not done:
        res = mmapwr.mmread_n_clear( mmapwr.MMAPRESP ) # read response
        print(f".../{inp}/==/{res}/..")
        if res==inp:
            break
        time.sleep(1)


def exclude(cmd=""):
    """
    certain protection from malicious shell string...
    """
    bad = False
    if cmd.find("&")>=0:  bad = True
    if cmd.find("|")>=0:  bad = True
    if cmd.find("'")>=0:  bad = True
    if cmd.find('$')>=0:  bad = True
    if cmd.find('%')>=0:  bad = True
    if cmd.find('#')>=0:  bad = True
    if cmd.find('!')>=0:  bad = True
    if cmd.find('(')>=0:  bad = True
    if cmd.find(')')>=0:  bad = True
    if cmd.find(';')>=0:  bad = True
    #if cmd.find('"')>=0:  die() # for sed

    if bad:
        print( f"{fg.white}{bg.red}X... not allowed char in {cmd}", fg.default,bg.default)
    return bad


#==========================================================
# this does classically expansion - more files.....
#==========================================================
def interpolate_star( parlis ):
    """
    Full cmd.split() ... only files, not directories; NOT USED
    """
    cmd2 = []
    newcmd = []
    newcmd.append(parlis[0])
    for i in parlis:
        cmd2.append(i)
    for i in range(1,len(cmd2)):
        print(">>>",cmd2[i] )
        if '*' in cmd2[i]:
            for j in glob.glob( cmd2[i] ):
                if not os.path.isdir(j):
                    newcmd.append(j)
        else:
            newcmd.append(cmd2[i])
    return newcmd
#==========================================================
# this does creates iteratoin through *  - more files.....
#==========================================================
def iterate_star( parstring ):
    """
    kw2.... only files, not directories; USED FOR LOCAL_FSl NOTUSED TOOO
    """
    #def replace_wildcard_with_files(string_A):
    parts = shlex.split(parstring)
    expanded_parts = []
    for part in parts:
        if '*' in part:
            expanded_parts.extend(glob.glob(part))
        else:
            expanded_parts.append(part)
    return [' '.join(expanded_parts)]

    # cmd2 = parstring.split()
    # newcmd = []
    # for i in cmd2:
    #     newcmd.append(i)
    #     if '*' in i:
    #         for j in glob.glob( cmd2[i] ):
    #             if not os.path.isdir(j):
    #                 newcmd.append(j)
    #     else:
    #         newcmd.append(cmd2[i])
    # return newcmd

# =========================================================
#   shell (True or False...check it) run of the commands. with some basic protection
# =========================================================
def run_or_die( cmd , debug = False):
    """
    runs shell command. Iterates over * from filesystem
    """
    res = 0
    if exclude(cmd): return
    res = 0
    #print()
    if debug: print("Exe...", cmd)
    cmd2 = cmd.split()
    for i in range(len(cmd2)):
        #print(i, cmd2[i])
        cmd2[i] = cmd2[i].strip('"')
    newcmd = []
    newcmd.append( cmd2[0] )
    for i in range(1,len(cmd2)):
        # print(">>>",cmd2[i] )
        if '*' in cmd2[i]:
            for j in glob.glob( cmd2[i] ):
                newcmd.append(j)
        else:
            newcmd.append(cmd2[i])
    #print(cmd2)
    if debug: print("Exe...",  newcmd)
    try:
        res = sp.check_call( newcmd )#, shell = True)
        if debug: print("ok",res)
    except:
        res =1
        print(f"X... {fg.red} error running /{bg.white}{cmd}{bg.default}/{fg.default}")
    #print()
    #if res != 0: die("")
# =========================================================

def termline(txt):
    #termsize3 = os.get_terminal_size().columns
    termsize3 = config.get_terminal_columns() # os.get_terminal_size().columns

    cont = f"#... ________ {txt} "
    cont = cont + "_"*(termsize3 - len(cont)-2)
    print(f"{fg.orange}{cont}{fg.default}")


# # ==============================================================================================
# # ==============================================================================================
# # ==============================================================================================
# # ==============================================================================================
# def load( spectrum = None):
#     """
#     special case, load
#     """
#     # Your code here
#     print("i... running command load INTERP", spectrum)
#     return f"loaded"

# def connect(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command connect  INTERP")
#     return f"conected"

# def unzoom(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command unzoom  INTERP")
#     return f"unzoomed"

# def zoom(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command zoom  INTERP")
#     return f"zoomed"
# # ==============================================================================================
# # ==============================================================================================
# # ==============================================================================================
# # ==============================================================================================


def main( cmd ):
    listcmd = cmd.split()
    kw1 = listcmd[0] #.split()[0]
    #
    # ======== I need to interpolate * for filesystem
    # if kw1 in KNOWN_COMMANDS_SHELL_FS or kw1 in KNOWN_COMMANDS_LOCAL_FS:
    #     print("D... FS interpol")
    #     listcmd = interpolate_star( listcmd) # only files, not directories
    kw2 = " ".join( listcmd[1:])
    #cmd = f"{kw1} {kw2}"
    #
    termline(cmd)
    match kw1:
        case 'reset':
            #print("RESET:",cmd,"    ")
            os.system("reset")
            move_cursor(3,1)
            return 1
        # attempt to use the interpretter here**********************
        case 'c':
             print(f"Command recognized /{cmd}/ ... connecting table   {list(kw2)} ")
             return 2
        # case 'zoom':
        #     #print("ZOOM:",cmd,"    ")
        #     return 2
        # case 'unzoom':
        #     #print("UNZOOM:",cmd,"    ")
        #     return 2
        # case 'connect':
        #     #print("CONNECT:",cmd,"    ")
        #     return 2
        case _: # =========== DEFAULT == SHELL COMMANDS ==========================
            # ====== SHELL COMMANDS (itteration of * from FS?)

            #if kw1 in KNOWN_COMMANDS_SHELL_OTHER:
            if kw1 in KNOWN_COMMANDS_SHELL_FS or kw1 in KNOWN_COMMANDS_SHELL_OTHER:
                # replace some commands
                if kw1=="ll": cmd = "ls -l "+kw2
                run_or_die(cmd)
                #

            elif kw1 in KNOWN_COMMANDS_LOCAL_OTHER: # zoom, show
                ### full control of cmd_parser ------- object list ----------
                ### func = getattr( cmd_parser, kw1) # WHEN ELSEWHERE
                ### --------- get the function from globals --------------------
                #func =  globals()[kw1] # WHEN HERE
                ### --------- HA! import a module fn_yrname
                func =  cmd_parser.str_func_to_func( kw1 )
                #func = func.main
                res = cmd_parser.call_function_with_command( func , kw2)
                # for i in res: print("RES:",i) # print results
                #

            elif kw1 in KNOWN_COMMANDS_LOCAL_OBJ: # zoom, show
                func =  cmd_parser.str_func_to_func( kw1 )
                res = cmd_parser.call_function_with_command( func , kw2)
                #

            elif kw1 in KNOWN_COMMANDS_LOCAL_FS:  # load
                ### ----- I need to do glob myself and call repetitively -------
                ### I try to import a module =======
                ###
                ### func = getattr( cmd_parser, kw1) # WHEN ELSEWHERE
                ### --------- get the function from globals --------------------
                #func =  globals()[kw1]
                ### --------- HA! import a module fn_yrname
                func =  cmd_parser.str_func_to_func( kw1 )
                # func = func.main
                # kw2_bis = iterate_star( kw2 )
                #for i in kw2_bis:
                res = cmd_parser.call_function_with_command( func , kw2, use_files=True)
                #for i in res: print("RES:",i) # print results

            else:
                print(f"{fg.red}X... unknown command /{cmd}/    {fg.default}")
            print("."*70)
            return 0   # 0 is the default case if x is not found
    pass
    #print()

if __name__=="__main__":
    Fire(main)



#!/usr/bin/env python3
#
# ----------  calling functions with parameters usin strings - way like Fire()
# done using GPT4
#
#
import shlex
import inspect
import fnmatch
from cronvice.version import __version__
from cronvice import config
from cronvice import objects
#from cronvice.config import object_list
from console import fg,bg
import glob
#
#import importlib_resources
import importlib
import sys
import os
#from importlib import import_module
#
# Assuming dflist is a global variable

#config.object_list = ['df1', 'df2', 'df3', 'd11', 'd21','h11']

#############################################################
#
#############################################################





# ---------------------------------------------------------------
def listpy( pretext="fn_", also_local = False, DEBUG = True):
    """
    return all pr_*.py files (in the current directory) (in repo dir)
    """
    files = []
    filescur = []
    dirnow = os.getcwd()
    # GO TO repo
    tgt2 = os.path.dirname(os.path.abspath(__file__))
    if DEBUG: print(f"{fg.dimgray}D... listpy is looking at {tgt2} {fg.default}")
    os.chdir( tgt2 )
    files = glob.glob(f"{pretext}*.py")
    if len(files)!=0:
        corenames = [ os.path.splitext(x)[0].split( pretext )[1] for x in files]
        #print(f"i... modules available in REPO:\n          {corenames}")
        files = [ tgt2+"/"+x for x in files ] # TGT2 DIR
        if DEBUG: print(f"D... {pretext}*py files: ",corenames)
    else:
        print(f"{fg.red}X... NO python FILES available here{fg.default}")
        print(f"{fg.red}X... installation is failing........... {fg.default}")
        sys.exit(1)

    files2 = []
    if also_local:
        if DEBUG: print(f"{fg.dimgray}D... looking at {dirnow} {fg.default}")
        os.chdir( dirnow )
        files2 = glob.glob(f"{pretext}*.py")
        if len(files2)!=0:
            corenames2 = [ os.path.splitext(x)[0].split( pretext )[1] for x in files2]
            #print(f"i... modules available in REPO:\n          {corenames}")
            files2 = [ dirnow+"/"+x for x in files2 ]  # DURRENT DIR
            if DEBUG: print(f"D... {pretext}*py files: ",corenames2)
        else:
            print(f"{fg.red}X... NO python FILES available here{fg.default}")
    return files+files2





#############################################################
#
#############################################################




# ---------------------------------------------------------------
def my_import_module(intit, pretext="fn_", also_local_files = False, DEBUG = True):
    """
    taken from prun.py - load (unload first) a module from the package (even local file)
    """

    files = listpy( also_local = also_local_files, DEBUG=DEBUG) # get all modules available
    if files is None:
        print(f"{fg.red}X... no loadable modules found....(@my_import_module){fg.default}")
        return None

    # print(files) # all present files
    # print(intit) # module
    module = None
    for ffile in files:
        # ffile may be with a path....
        construct = ffile
        if len(ffile.split("/"))>1:
            construct = construct.split("/")[-1]
        construct =  construct.lstrip(pretext)
        # print(construct)
        construct =  construct.rstrip("py")[:-1] # prioblem w .py
        # print(construct)
        ##print(f" ... searching /{intit}/ in /{ffile}/ <={construct}")
        # this was the last searching
        #print(f" ... searching /{intit}/ in {ffile} ")

        if (intit == construct):
            #print(f"P... got {ffile}")
            #print("i... trying to importlib:", item.GetTitle() )
            # UNimport module first - case there was an error there previously
            unloadname = f"{pretext}{intit}"
            if DEBUG: print(f"{fg.dimgray}D... module /{unloadname}/ ... {fg.default}", end="")
            try:
                sys.modules.pop( unloadname ) # ffile[:-3]     f"pr_{construct}")
                if DEBUG: print(f"...{fg.dimggray}... unloaded successfully, loading now...{fg.default}")
            except:
                if DEBUG: print(f"...{fg.dimgray}... not unloaded, loading now...{fg.default}")
            # IMPORT
            #print("D... importing module")
            #module = importlib.import_module( ffile[:-3] ) # older
            spec = importlib.util.spec_from_file_location( pretext+intit, ffile )
            if DEBUG: print(f"D...{fg.dimgray}... {spec}   ...{fg.default}")
            module = importlib.util.module_from_spec(spec)
            if DEBUG: print(f"D...{fg.dimgray}... {module}  ...{fg.default}")
            spec.loader.exec_module(module)
            if DEBUG: print(f"{fg.dimgray}D... importing module DONE{fg.default}")
            #break

    return module
# ---------------------------------------------------------------




#############################################################
#
#############################################################





#############################################################
#
#############################################################

def str_func_to_func(sfunc = ""):
    """
    """
    # current_module1 = sys.modules[__name__]
    # print(current_module1)
    # current_module2 = __import__(__name__)
    # print(current_module2)

    #this_pkg = sys.modules[__package__]
    #print(this_pkg)

    if len(sfunc)=="":
        print(f"{fg.red}X... no function name given for search... {fg.default}")
        return None

    # mod2search = f"fn_{sfunc}"
    current_module3 = my_import_module(sfunc,  pretext="fn_",
                                       also_local_files=True,
                                       DEBUG=False)
    # print(current_module3)
    #if current_module3 is not None:
    #    current_module3.main()
    return current_module3.main


#############################################################
#
#############################################################

def call_function_with_command(func, command, use_files=False, DEBUG=True):
    """
    provide "func" as the real function and inspect its parameter set
    """
    tokens = shlex.split(command)
    ok = False
    try:
        sig = inspect.signature(func)
        #print("D... these are the revealed parameters... ",sig)
        ok = True
    except:
        print(f"{fg.red}X... NO FUNCTION seen like {fg.default}",func,"...",command)
        ok = False
    if not ok : return []
    args = []
    kwargs = {}
    results = []

    if len(tokens)==0:
        results.append( func() )
        return results

    it = iter(tokens)
    print(f"{fg.dimgray}D... tokens:{fg.default}", tokens)
    for token in it:
        if token.endswith('*') or '*' in token:
            # Handle the pattern matching by deferring execution until later
            args.append(token)
        elif token.startswith('-'):
            # Remove the '-' prefix
            flag = token.lstrip('-')
            value = next(it, None)
            # Attempt to match the flag to known parameters
            matched_params = [p for p in sig.parameters if p.startswith(flag)]
            if not matched_params:
                print(f"X... ERROR - unknown flag {token}")
                return []
                raise ValueError(f"Unknown flag: {token}")
            if len(matched_params) > 1:
                print(f"X... ERROR - ambiguous flag {token}")
                return []
                raise ValueError(f"Ambiguous flag: {token}")
            param_name = matched_params[0]
            param = sig.parameters[param_name]
            # Convert the value to the correct type based on the default value
            if param.default is not inspect.Parameter.empty and param.default is not None:
                if isinstance(param.default, bool):
                    value = value.lower() in ('true', 'yes', '1')
                else:
                    value = type(param.default)(value)
            elif param.default is None:
                # If the default is None, we keep the value as a string
                pass
            kwargs[param_name] = value
        else:
            if DEBUG: print(f"{fg.dimgray}D... appending {fg.default}", token)
            args.append(token)

    for pattern in args:
        if DEBUG: print(f"{fg.dimgray}D... working on arg:/{pattern}/   {fg.default} ")
        matching_dfs=[]
        # Run the function for each dataframe in config.object_list that matches the pattern

        if use_files:
            if DEBUG: print(f"{fg.dimgray}D... interpreting pattern as a file /{pattern}/  :{fg.default}")
            matching_dfs = glob.glob( pattern )
            if len(matching_dfs)==0:
                print(f"{fg.red}X... no files match /{pattern}/{fg.default}")

        elif objects.object_exists( pattern ): #pattern in objects.get_objects_list():
            if DEBUG: print(f"{fg.dimgray}D... interpreting pattern as an object /{pattern}/  :{fg.default}")
            matching_dfs = fnmatch.filter(objects.get_objects_list_names(), pattern)
            if len(matching_dfs)==0:
                print(f"{fg.red}X... no objects match /{pattern}/{fg.default}")
            elif DEBUG:
                print(f"{fg.dimgray}D... IT IS  in the list of allowed objects{fg.default}   ")
        else:
            print(f"{fg.red}X... argument /{pattern}/ IS NOT in the list of allowed objects: {objects.get_objects_list()}  {fg.default}"  )
        #
        #
        for dfname in matching_dfs:
            # Substitute '$' with the dataframe name for all kwargs
            #print(f"i... running FUNC with name {dfname}")
            modified_kwargs = {k: v.replace('$', dfname) if isinstance(v, str) and '$' in v else v
                               for k, v in kwargs.items()}
            # print(f"i... kwargs: ", modified_kwargs)
            #try:
            RES = func(dfname, **modified_kwargs)
            results.append(RES)
            #except:
            #    print(f"X... {fg.red}function {func}({command}) crashed {fg.default} ")
    # print(".")
    return results




# # Example function
# def cut(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command cut")
#     return f"cut({dfname}, from_={from_}, to={to}, display={display}, savename='{savename}' quest='{quest}')"


# ==============================================================================================
# ==============================================================================================
# ==============================================================================================
# ==============================================================================================
# def load( spectrum = None ):
#     # Your code here
#     print("i... running command load")
#     return f"loaded"

# def connect(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command connect")
#     return f"conected"

# def unzoom(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command unzoom")
#     return f"unzoomed"

# def zoom(dfname, from_=0, to=999999, display=False, savename=None, quest="meo"):
#     # Your code here
#     print("i... running command zoom")
#     return f"zoomed"




if __name__ == "__main__":
    # Example usage:
    command = "d*1 -f 10 -t 20 -s bis_$.txt"
    command = "d*1 -f 10 -t 20 -q 'Hi! $_bis; ls $_bis' -s $.txt"
    results = call_function_with_command(cut, command)
    for result in results:
        print(result)  # Output will be the string representation of the function call for each matching dataframe
    str_func_to_func("load")



#!/usr/bin/env python3
#
#  for development :
# pip3 install --use-pep517  -e . --upgrade  from virtual env.
#
import os
from setuptools import setup
#from fire import Fire

#-----------problematic------
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

import os.path

def readver(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in readver(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            version = line.split(delim)[1]
            print("i... setup.py: Version found:", version)  # Debug print
            return version
    else:
        raise RuntimeError("Unable to find version string.")

#if __name__ == "__main__":
#    print( get_version("cronvice/version.py")   )

setup(
    name="cronvice",
    description="Automatically created environment for python package",
    author="me",
    url="https://gitlab.com/ojr/cronvice",
    author_email="ojr@gmail.com",
    license="GPL2",
    version=get_version("cronvice/version.py"),
    packages=['cronvice'],
    package_data={"cronvice":["data/testik"]},
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    scripts = ['bin/cronvice'],
    install_requires = ['fire','console','sshkeyboard','pytermgui','blessings','terminaltables','pandas','pyfiglet','prompt_toolkit', 'python-crontab'],
)



#!/usr/bin/env python3

# from {proj}.version import __version__
import time
import datetime as dt
import os
from fire import Fire
import pytermgui # import  report_cursor, save_cursor, restore_cursor
from cronvice import config
# import threading  # for key input

from console import fg, bg, fx
import socket
import hashlib
# theight= terminal.height
# twidth= terminal.width

global_mode = " "

    #==================================================================
    #
    #------------------------------------------------------------------

def get_hostname():
    return socket.gethostname()

    #==================================================================
    #
    #------------------------------------------------------------------

def get_deterministic_index():
    termcodes = {}
    termcodes["slateblue"] = 62
    termcodes["brown"] = 124
    termcodes["purple"] = 5
    termcodes["darkcyan"] = 30
    termcodes["cadetblue"] = 73
    termcodes["darkslateblue"] = 60
    termcodes["darkgreen"] = 22
    termcodes["sienna"] = 130
    termcodes["indianred"] = 167
    termcodes["seagreen"] = 29
    hostname = socket.gethostname()
    hash_value = hashlib.sha256(hostname.encode()).hexdigest()[0:8]
    #print(hash_value)
    index = int(hash_value, 16) % len(termcodes)  # Get deterministic index
    #print("index", index)
    return index # int(hash_value, 16) % 10

    #==================================================================
    #
    #------------------------------------------------------------------

def get_deterministic_color_term():
    """
    This is consistent with bash script in tjkirchmod2 !!!!
    #https://www.w3.org/TR/css-color-3/#svg-color

    """
    termcodes = {}
    termcodes["slateblue"] = 62
    termcodes["brown"] = 124
    termcodes["purple"] = 5
    termcodes["darkcyan"] = 30
    termcodes["cadetblue"] = 73
    termcodes["darkslateblue"] = 60
    termcodes["darkgreen"] = 22
    termcodes["sienna"] = 130
    termcodes["indianred"] = 167
    termcodes["seagreen"] = 29

    hostname = socket.gethostname()
    hash_value = hashlib.sha256(hostname.encode()).hexdigest()[0:8]
    print(hash_value)
    index = int(hash_value, 16) % len(termcodes)  # Get deterministic index
    print("index", index)
    color_name = list(termcodes.keys())[index]  # Map index to a color name
    return termcodes[color_name]  # Return the xterm-256 color index


    #==================================================================
    #
    #------------------------------------------------------------------

class TopBar:
    """
    allows to define top bar(s) and keep printing them
    """

    host_colors =  [ bg.slateblue,   bg.brown, bg.purple,   bg.darkcyan, bg.cadetblue, bg.darkslateblue, bg.darkgreen,  bg.sienna,  bg.indianred, bg.seagreen]
    # not used, just in findcolor
    cocodes = {}
    cocodes["slateblue"] = "#6A5ACD"
    cocodes["brown"] = "#A52A2A"
    cocodes["purple"] = "#800080"
    cocodes["darkcyan"] = "#008B8B"
    cocodes["cadetblue"] = "#5F9EA0"
    cocodes["darkslateblue"] = "#483D8B"
    cocodes["darkgreen"] = "#006400"
    cocodes["sienna"] = "#A0522D"
    cocodes["indianred"] = "#CD5C5C"
    cocodes["seagreen"] = "#2E8B57"


    #==================================================================
    #
    #------------------------------------------------------------------

    def __init__(self, pos=1, bgcolor="auto"):
        self.pos = pos # bar number 1 (most top) or more
        self.elements = {}
        self.twidth = 80 # terminal width online
        #self.positions = {}
        self.t2 = None
        if bgcolor is None:
            self.BCOL = bg.blue
        elif bgcolor == "auto":
            i = get_deterministic_index()
            self.BCOL = self.host_colors[i]
        else:
            self.BCOL = bgcolor

    #==================================================================
    #
    #------------------------------------------------------------------

    @classmethod
    def get_colors(cls):
        return cls.host_colors

        # self.t = threading.currentThread()

        # try:
        #     pass
        #     # print("report_cursor to appear")
        #     # print( "i... topbar: pos/cursor",pos  )
        #     report_cursor()
        #     # print("report done")
        # except:
        #     print("X... problem with report_cursor")
        # # print("i... topbar bar started")

    #==================================================================
    #
    #------------------------------------------------------------------

    def add_bar(self, two=2, bgcolor=bg.blue):
        """
        create second bar
        """
        if two == 2:
            self.t2 = TopBar(two, bgcolor=bgcolor)
        else:
            print("X... nobody wanted more than two......  NOT OK")
        return self.t2

    #==================================================================
    #
    #------------------------------------------------------------------

    def add_element(self, name, x, length, text, style):
        """
        insert into the bar, x is x coordinate, no tuple anymore; tup<0 is from right
        """
        xp = 1
        if isinstance(x,int):
            xp = x
        else:
            print("X... only  int in the TOPBAR  for position")
            sys.exit(1)
        if type(text) != str:
            print("X... display string only")
            sys.exit(1)
        #self.positions[x] = s # coordinate has text
        self.elements[name] = [ x, length, text, style] # x coordinate  ; text  text
        #print(self.elements.keys() )

    def update_element(self, name, text, style=None):
        if name in self.elements:
            x = self.elements[name][0]
            length = self.elements[name][1]
            style1 = self.elements[name][3]
            if style is not None:
                style1 = style
            self.elements[name] = [ x, length, text, style1] # x coordinate  ; text  text


    #==================================================================
    #
    #------------------------------------------------------------------
    def prepare_element(self, eledict):
        """
        prepare elements list and size, always 3x default @end
        """
        x0 = eledict[0]
        length = eledict[1]
        text = eledict[2]
        style = eledict[3]
        #
        if len(text) > length:
            text = text[:length]
        elif len(text) < length:
            if x0 < 0:
                text = text.rjust(length)
            else:
                text = text.ljust(length)
        text = f"{self.BCOL}{style}{text}{bg.default}{fx.default}{fg.default}"
        if x0 < 0:
            x0 = self.twidth - abs(x0) + 1
            #if x0 + length > self.twidth:
            #    text = text[: self.twidth - x0 - length]
        if x0 < 0:
            x0 = 1
        return x0, text


    #==================================================================
    #
    #------------------------------------------------------------------

    def place(self):
        """
        Place the TOPBAR on screen
        """
        # curs = (-1, -1)
        #twidth = os.get_terminal_size().columns
        self.twidth = config.get_terminal_columns() # os.get_terminal_size().columns
        #print(twidth)

        if self.pos == 1:
            pytermgui.save_cursor()

        # do background first -------------------------------
        pytermgui.print_to( (1, self.pos), f"{self.BCOL}" + " " * self.twidth + bg.default) # paint default BG
        pytermgui.print_to( (1, self.pos + 1), " " * self.twidth)

        # --------------------------------------------------------------------------------
        #
        for k in self.elements.keys():
            x0, text = self.prepare_element( self.elements[k]  )
            pytermgui.print_to(   ( x0, self.pos), text )

        if self.t2 is not None:
            self.t2.place()

        if self.pos == 1:
            pytermgui.restore_cursor()
            print("", end="\r")  # this make GOOD thing in printing



# #################################################################################3
#
#
#
# ------------------------------------------------------------------------------------
def main():
    """
    print an example top bar
    """

    # BOTH
    # bg.cadetblue  bg.steelblue bg.darkgreen, bg.olive
    # bg.steelblue, is too close
    print("\n\n showing some  bar colors ..... \n\n")
    for c in [ bg.navy, bg.steelblue]:
        print(fg.white, c, "        SEE an example of a top bar  -  ",fg.black, "kill with  Ctrl-c     ", bg. default)


    print()
    print(fg.white, "with  WHITE-and-BLACK LETTERS I have now :  ***10***  useful colors !") # bg.olive,

    for c in TopBar.get_colors():
        print(fg.white, c, "        SEE an example of a top bar  -  ",fg.black, "kill with  Ctrl-c     ", bg. default, fg.default)

    # print(fg.white, "with  BLACK LETTERS")
    # for c in [  bg.orange, bg.khaki, bg.burlywood, bg.aquamarine, bg.sandybrown, bg.tomato, bg.salmon, bg.lightcoral,]:
    #     print(fg.black, c, "        SEE an example of a top bar  -  ",fg.white, "kill with  Ctrl-c     ", bg. default)

    #time.sleep(3)
    print("-------------------------------------------")
    print(" #       add_element( NAME, x (positive or negative),  length,   TEXT, tyle  ) ")
    print(" #       update_element( NAME,  TEXT  ) ")
    print()
    t = TopBar(1)
    t.add_element("time", 11,10 + 12, str(dt.datetime.now())[:-4], fg.white + fx.bold)
    t.add_element("host", -10,10, get_hostname(), bg.orange + fx.bold + fg.blue)
    #
    print("\n\n  see the bar running for 33 seconds \n\n")
    for i in range(100):
        #
        # DO whatever stuff and PLACE PRINTTO SLEEP
        #
        t.place()
        t.update_element("time", str(dt.datetime.now())[:-4] )
        #t.update_element("host", get_hostname() )
        time.sleep(0.3)



if __name__ == "__main__":
    Fire(main)
    #Fire({"termcolor":get_deterministic_color_term,       "m":  main})
    #Fire(get_deterministic_color_term)
